/*
    File:             onemediumc.cpp
    Author:           Tomas Krejci
    Purpose:          computes conductivity and capacity matrices in a material point for coupled three media trasport with mechanics
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "onemediumc.h"
#include "coupmatu.h"
#include "coupmatl.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "isotrmatc.h"
#include "intpoints.h"
#include "consol_awf1.h"

medc1::medc1()
{  
  /* scale_t = Tp->scale[0];
     scale_u = Mp->scale;
  */
  //not completed
  scale_t = 1.0;
  scale_u = 1.0;
}
medc1::~medc1()
{}


/**
   function computes conductivity matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - conductivity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcond_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double k;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      matrix sd(d.m,d.n);
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      k = Cmu->itrmc[i].get_alpha();//*scale_u;//scaling
      k = -1.0*k;
      
      switch (m){
      case 1:{//1D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	break;
      }
      case 3:{//2D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	sd[1][0] = k;
	sd[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	fillm(0.0,sd);
	
	sd[0][0]=k;  
	sd[1][0]=k;   
	sd[2][0]=k;
	sd[3][0]=0.0;   
	sd[4][0]=0.0;  
	sd[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      matrix s(d.m,d.m);
      
      Cmu->matstiff (s,ipp);//only for temperature
      mxm(s,sd,d);
      
      destrm (s); destrm (sd);
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  case mech_water:{
    switch (Cmu->ip[ipp].tm){//material type
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kuw(pw);//*scale_u;//scaling
      break;
    }
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->consol_awf1c[i].get_kuw(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    switch (m){
    case 1:{//1D
      d[0][0] = k;
      break;
    }
    case 3:{//2D
      d[0][0] = k;
      d[1][0] = k;
      d[2][0] = 0.0;  
      break;
    }
    case 6:{//3D
      d[0][0]=k;  
      d[1][0]=k;   
      d[2][0]=k;
      d[3][0]=0.0;   
      d[4][0]=0.0;  
      d[5][0]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}




/**
   function computes capacity matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - cpapcity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcap_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double c;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      
      c = 0.0;
      
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->sejtkrmc[i].get_capuw(pw);//*scale_u;//scaling
      break;
    }
      
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->consol_awf1c[i].get_capuw(pw);//*scale_u;//scaling
      break;
    }

    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    
    switch (m){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = c;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = c;
      d[1][0] = c;
      d[2][0] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=c;  
      d[1][0]=c;   
      d[2][0]=c;
      d[3][0]=0.0;   
      d[4][0]=0.0;  
      d[5][0]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes conductivity matrix D in a material point for one medium transfer
   for lower block
   
   @param d   - conductivity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcond_l (matrix &d,long ri,long ci,long ipp)
{
  long i,n;
  double k;
  n = d.n;
  
  i = Cml->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cml->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cml->ip[ipp].av[0]*scale_t;//scaling
      k = 0.0;
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kwu(pw);//*scale_u;//scaling
      break;
    }
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->consol_awf1c[i].get_kwu(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    switch (n){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = k;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = k;
      d[0][1] = k;
      d[0][2] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=k;  
      d[0][1]=k;   
      d[0][2]=k;
      d[0][3]=0.0;   
      d[0][4]=0.0;  
      d[0][5]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}




/**
   function computes capacity matrix D in a material point for one medium transfer
   for lower block
   
   @param d   - cpapcity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcap_l (matrix &d,long ri,long ci,long ipp)
{
  long i,n;
  double c;
  n = d.n;
  
  i = Cml->ip[ipp].idm;

  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cml->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cml->ip[ipp].av[0]*scale_t;//scaling
      
      c = 0.0;
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->sejtkrmc[i].get_capwu(pw);//*scale_u;//scaling
      break;
    }
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->consol_awf1c[i].get_capwu(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    } 
        
    switch (n){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = c;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = c;
      d[0][1] = c;
      d[0][2] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=c;  
      d[0][1]=c;   
      d[0][2]=c;
      d[0][3]=0.0;   
      d[0][4]=0.0;  
      d[0][5]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function computes right-hand-side matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - right-hand-side %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::rhs_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double k;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:{
    switch (Cmu->ip[ipp].tm){//material type
    case consolawf1c:{
      Cmu->consol_awf1c[i].rhs_volume(d,ri,ci,ipp);
      break;
    }
    case sejtkrc:{
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }	  
    break;
  }
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      matrix sd(d.m,d.n);
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      
      k = Cmu->itrmc[i].get_alpha();//*scale_u;//scaling
      k = -1.0*k;
      
      switch (m){
      case 1:{//1D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	break;
      }
      case 3:{//2D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	sd[1][0] = k;
	sd[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	fillm(0.0,sd);
	
	sd[0][0]=k;  
	sd[1][0]=k;   
	sd[2][0]=k;
	sd[3][0]=0.0;   
	sd[4][0]=0.0;  
	sd[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      matrix s(d.m,d.m);
      
      Cmu->matstiff (s,ipp);//only for temperature
      mxm(s,sd,d);
      
      destrm (s); destrm (sd);
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}
